import { ObserverCallback } from "@hibas123/utils";
import { ICollectionRef } from "./types";
declare type IWriteQueries = "set" | "update" | "delete" | "add";
declare type ICollectionQueries = "get" | "add" | "keys" | "delete-collection" | "list";
declare type IDocumentQueries = "get" | "set" | "update" | "delete";
declare type ICallQueryTypes = IWriteQueries | ICollectionQueries | IDocumentQueries | "snapshot" | "unsubscribe";
interface ITypedQuery<T> {
    path: string[];
    type: T;
    data?: any;
    options?: any;
}
declare abstract class Snapshot<T> {
    protected id: string;
    isSubscribed: boolean;
    abstract subscribe(): Promise<() => void>;
    abstract unsubscribe(): void;
    abstract receivedData(data: T): void;
    abstract resubscribe(): void;
    abstract destroy(): void;
}
declare const QueryRequest: unique symbol;
declare const Snapshots: unique symbol;
export declare class OfflineError extends Error {
    constructor();
}
export declare class AwaitStore<T> {
    private _value;
    private observable;
    constructor(_value: T);
    get value(): T;
    send(value: T): void;
    subscribe(handler: ObserverCallback<T>): void;
    unsubscribe(handler: ObserverCallback<T>): void;
    awaitValue(val: T): PromiseLike<void> & {
        catch: (cb: (err: any) => PromiseLike<void>) => PromiseLike<void>;
        ignore: () => void;
    };
    close(): void;
}
export default class Database {
    static WebSocket: typeof WebSocket;
    private offlineObersvable;
    get offline(): {
        subscribe: (handler: ObserverCallback<boolean>) => void;
        unsubscribe: (handler: ObserverCallback<boolean>) => void;
    };
    private socket;
    private requests;
    private url;
    constructor(url: string, database: string, accesskey?: string, authkey?: string, rootkey?: string);
    private _connect;
    [Snapshots]: Map<string, Snapshot<any>>;
    private handleSnapshot;
    private handleResponse;
    [QueryRequest]<T = any>(query?: ITypedQuery<ICallQueryTypes>, id?: string, ns?: string): Promise<T>;
    collection<T = any>(name: string): ICollectionRef<T>;
    collections(): Promise<ICollectionRef[]>;
    close(): void;
}
export {};
