import DB, { IDocumentRef, UpdateTypes } from "./node";

const log = (pref: string, prms: Promise<any>) => prms.then(val => console.log(pref + ": ", val));


(async () => {
   const db = new DB("http://localhost:5000", "test_database", "access87125t487123897458725123", undefined, "root12341213312312365465as4df6");
   console.log("Start!");
   let coll1 = db.collection("test1");

   await coll1.delete(); //Clean the Database

   await log("COLL1 keys", coll1.keys().then(() => "Make sure to clear database before testing!").catch(() => "Successfully thrown an error on not existing collection"));
   let doc1: IDocumentRef;
   await log("Adding new document with push", coll1.add({ name: "Max Mustermann" }).then(res => doc1 = res).then(res => res.id));
   let doc2 = coll1.doc();
   await log("Adding new document with doc().set(...)", doc2.set({ name: "Franz Mustermann" }));
   await log("COLL1 keys", coll1.keys());

   await log("Getting first added document", doc1.get())

   await log("Override first added document", doc1.set({ name: "Lisa Mustermann" }))

   await log("Getting first added document", doc1.get())

   await log("Deleting new document", doc2.delete());

   await log("Getting first added document", doc2.get())

   const snap1 = await coll1.onSnapshot((err, change) => {
      if (!err) {
         console.log(change.docChanges().map(ch => [ch.type, ch.doc.data()]))
      } else {
         console.log(err);
      }
   })

   await log("Setting Doc2", doc2.set({ name: "Micha Ekkehardt", age: 1, s: { t: 0 }, bands: [] }));

   await log("Incrementing age", doc2.update({ age: UpdateTypes.Increment(2) }));
   await log("Incrementing age 2", doc2.update({ age: UpdateTypes.Increment(2) }));
   await log("Incrementing s->t", doc2.update({ "s.t": UpdateTypes.Increment(2) }));
   await log("Incrementing s->r 2", doc2.update({ "s.r": UpdateTypes.Increment(2) }));
   await log("Incrementing g->r 2", doc2.update({ "g.r": UpdateTypes.Increment(2) }));

   await log("Pushing into band array", doc2.update({ bands: UpdateTypes.Push("Fall Out Boy") }))
   await log("Pushing into movies array", doc2.update({ movies: UpdateTypes.Push("Matrix") }))

   await log("Setting timestamp into field timestamp", doc2.update({ timestamp: UpdateTypes.Timestamp }))

   await new Promise((resolve) => setTimeout(resolve, 200));

   snap1();

   await log("Deleting doc2", doc2.delete());

   console.log("Adding 100 docs");
   console.time("adding")
   for (let i = 0; i < 100; i++) {
      await coll1.add({
         index: i
      })
   }
   console.timeEnd("adding")
   console.log("Added 100 docs");

   await log("Getting docs with index larger than 95", coll1.where("index", ">", 95).get().then(res => res.docs.map(d => d.data().index)));
   await log("Getting docs with index larger or equal than 95", coll1.where("index", ">=", 95).get().then(res => res.docs.map(d => d.data().index)));
   await log("Getting docs with index equal to 95", coll1.where("index", "==", 95).get().then(res => res.docs.map(d => d.data().index)));
   await log("Getting docs with index smaller  or euqal than 5", coll1.where("index", "<=", 5).get().then(res => res.docs.map(d => d.data().index)));
   await log("Getting docs with index smaller than 5", coll1.where("index", "<", 5).get().then(res => res.docs.map(d => d.data().index)));

   await log("Getting docs smaller than 5 in ascending order", coll1.where("index", "<", 5).orderBy("index", "asc").get().then(res => res.docs.map(d => d.data().index)));
   await log("Getting docs smaller than 5 in descending order", coll1.where("index", "<", 5).orderBy("index", "desc").get().then(res => res.docs.map(d => d.data().index)));

   await coll1.add({
      bands: ["Fall Out Boy"]
   })

   await coll1.add({
      bands: ["Panic at the Disco"]
   })

   await log("Getting docs with band: Panic at the Disco", coll1.where("bands", "array-contains", "Panic at the Disco").get().then(res => res.docs.map(d => d.data().bands)));

   const snap2 = await coll1.where("bands", "array-contains", "Panic at the Disco").onSnapshot((err, change) => {
      if (!err) {
         console.log(change.docChanges().map(ch => [ch.type, ch.doc.data().bands]))
      } else {
         console.log(err);
      }
   })

   let doc3: IDocumentRef;
   await log("Adding doc with Fall Out Boy", coll1.add({ bands: ["Fall Out Boy"] }).then(r => { doc3 = r }))
   await log("Adding doc with Panic at the Disco", coll1.add({ bands: ["Panic at the Disco"] }).then(() => undefined))

   await log("Adding Panic at the Disco to document", doc3.update({ bands: UpdateTypes.Push("Panic at the Disco") }))

   await new Promise((resolve) => setTimeout(resolve, 200));

   db.close();
})()
   .catch(err => console.error(err))
   .then(() => process.exit())